#!/usr/bin/perl -w

# Copyright (c) 2003 Red Hat
# Written by Graydon Hoare <graydon@redhat.com>

# This file is part of Mauve.

# Mauve is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.

# Mauve is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Mauve; see the file COPYING.  If not, write to
# the Free Software Foundation, 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.  */

# Choose the tests we want to run.
# Usage: choose.pl output-directory [tag] ...
# Run in the source directory.
# See README for information on tags.

use File::Find;
use Carp;
use strict;

my %tests_to_run = ();
my %tags_to_find = ();
my $basedir = 'gnu/testlet';
my @dirs = qw( java javax BinaryCompatibility );
my @file_patterns = ();

sub scan_deps
{
    my $filename = shift(@_);
    return if (exists($tests_to_run{$filename}));
    $tests_to_run{$filename} = 1;

    my @deps = ();
    open(FILE, $filename);
    while (<FILE>)
    {
	if (m@^// Uses: (.*)$@o)
	{
	    my $uses = $1;
	    my @uses = split(/\s+/, $uses);
	    for my $used (@uses)
	    {
		my $target = $File::Find::dir . '/' . $used . '.java';
		-e $target || croak("missing 'Uses:' target $target, in $filename");
		push @deps, $target;
	    }
	}
    }
    close(FILE);

    for my $dep (@deps)
    {
	scan_deps($dep);
    }
}

sub finder
{
    if ($File::Find::name =~ /.java$/o)
    {
	my $filename_matches = 0;
	my $tag_spec_matches = 0;

	if (not @file_patterns)
	{
	    # no file patterns -> everything matches
	    $filename_matches = 1;
	}
	else
	{
	    my $simple_name = $File::Find::name;
	    $simple_name =~ s@^$basedir/@@o;
	    for my $pat (@file_patterns)
	    {
		if (('!' . $simple_name) =~ /^$pat/)
		{
		    $filename_matches = 0;
		}
		elsif ($simple_name =~ /^$pat/)
		{
		    $filename_matches = 1;
		}
	    }
	}

	if ($filename_matches)
	{
	    if (not %tags_to_find)
	    {
		# no tag patterns -> everything matches
		$tag_spec_matches = 1;
	    }
	    else
	    {
		open(FILE, $File::Find::name);
		while (<FILE>)
		{
		    if (m@^// Tags: (.*)$@o)
		    {
			my $tags = $1;
			my @tags = split(/\s+/, $tags);
			for my $tag (@tags)
			{
			    if ($tag eq 'not-a-test')
			    {
				close(FILE);
				return;
			    }
			}
			
			my $included_by_tag = 0;
			my $excluded_by_tag = 0;
			
			for my $tag (@tags)
			{
			    if (length($tag) > 2 
				&& substr($tag,0,1) eq '!'
				&& exists($tags_to_find{substr($tag,1)}))
			    {
				$excluded_by_tag = 1;
			    }
			    elsif (exists($tags_to_find{$tag}))
			    {
				$included_by_tag = 1;
			    }
			}
			if ($included_by_tag && (! $excluded_by_tag))
			{
			    $tag_spec_matches = 1;
			}
		    }
		}
		close(FILE);
	    }
	}

	# scan dependencies of this file
	if ($tag_spec_matches)
	{
	    scan_deps($File::Find::name);
	}
    }
}

# main()

croak "usage: choose <OUTDIR> [KEYS...]" unless @ARGV;

my $outdir = shift(@ARGV);
for my $arg (@ARGV)
{
    if ($arg =~ /java/)
    {
	$arg =~ s@\.@/@;
	push @file_patterns, $arg;
    }
    else
    {    
	$tags_to_find{$arg} = 1;
    }
}

# massage arguments
if (not %tags_to_find
    || exists $tags_to_find{'JDK1.1'} 
    || exists $tags_to_find{'JDK1.2'})
{
    $tags_to_find{'JDK1.1'} = 1;
    $tags_to_find{'JDK1.0'} = 1;
}

# search directories
my @qualified_dirs = map { $basedir . '/' . $_; } @dirs;
find({ 'no_chdir' => 1,
       'wanted' => \&finder },
     @qualified_dirs);


# output results 
my $test = '';
open (CLASSES, ">$outdir/classes");
for $test (sort(keys(%tests_to_run)))
{
    $test =~ s/.java$//o;
    $test =~ s@/@.@go;
    print CLASSES $test, "\n";
}
close (CLASSES);


open (CHOICES, ">$outdir/choices");
print CHOICES 'CHOICES=';
for $test (sort(keys(%tests_to_run)))
{
    print CHOICES $test, " \\\n";
}
print CHOICES "\n";

print CHOICES 'SimpleTestHarness: ';
for $test (sort(keys(%tests_to_run)))
{
    $test =~ s/.java$/.o/o;
    print CHOICES $test, " \\\n";
}
print CHOICES "\n";
close (CHOICES);




